import React, { createElement } from 'react';
import { getCharacterEntityType, linkify as _linkify, } from '../@utils';
const LinkComponent = ({ href, children, }) => React.createElement("a", { href: href }, children);
export function createLinkDecorator({ component = LinkComponent, rules, }) {
    const linkify = _linkify(rules);
    const Link = ({ contentState, decoratedText, entityKey, children, }) => {
        let href;
        if (entityKey) {
            ({ href } = contentState.getEntity(entityKey).getData());
        }
        else {
            ({ url: href } = linkify.match(decoratedText)[0]);
        }
        return createElement(component, { contentState, decoratedText, entityKey, href }, children);
    };
    return {
        component: Link,
        strategy(block, callback, contentState) {
            let entityRanges = [];
            block.findEntityRanges(metadata => getCharacterEntityType(metadata, contentState) === 'LINK', (start, end) => {
                entityRanges.push([start, end]);
                callback(start, end);
            });
            let characters = Array.from(block.getText());
            for (let [start, end] of entityRanges) {
                let length = end - start;
                characters.splice(start, end - start, ''.padEnd(length));
            }
            let text = characters.join('');
            let links = linkify.match(text);
            if (links) {
                for (let link of links) {
                    callback(link.index, link.lastIndex);
                }
            }
        },
    };
}
//# sourceMappingURL=link-decorator.js.map